package com.kms.handler;

import com.kms.crosscut.SoapFaultHelper;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Set;

public class CacheHandler implements SOAPHandler<SOAPMessageContext> {

    private static final String REDIS_HOSTNAME =
            System.getenv().getOrDefault("REDIS_HOSTNAME", "localhost");
    private static final JedisPool jedisPool = new JedisPool(REDIS_HOSTNAME, 6379);
    private static final int DEFAULT_CACHE_TTL = 300;

    @Override
    public Set<QName> getHeaders() {
        return null;
    }

    @Override
    public boolean handleMessage(SOAPMessageContext context) {
        final boolean messageHandled =
                (boolean) context.getOrDefault("messageHandled-Cache", false);

        if (!messageHandled) {
            context.put("messageHandled-Cache", true);
            return true;
        }

        final boolean cacheResponse =
                (boolean) context.getOrDefault("cacheResponse", false);
        if (cacheResponse) {
            final String cacheKey = (String) context.getOrDefault("cacheKey", null);
            if (cacheKey == null)
                return true;

            try (Jedis jedis = jedisPool.getResource()) {
                OutputStream responseTextOStream = new ByteArrayOutputStream();
                context.getMessage().writeTo(responseTextOStream);

                jedis.set(cacheKey, responseTextOStream.toString());
                jedis.expire(cacheKey, DEFAULT_CACHE_TTL);

                System.out.println("Response cached with key: " + cacheKey);
            } catch (SOAPException | IOException ignored) {
            }

            return true;
        }

        final boolean returnFromCache =
                (boolean) context.getOrDefault("returnFromCache", false);
        if (!returnFromCache)
            return true;

        final String cacheKey = (String) context.getOrDefault("cacheKey", null);
        if (cacheKey == null) {
            SoapFaultHelper.throwFault("Error", "Unexpected error has happened.");
        }
        try (Jedis jedis = jedisPool.getResource()) {
            String response = jedis.get(cacheKey);
            SOAPMessage soapMessage = MessageFactory.newInstance().createMessage();
            soapMessage.getSOAPPart().setContent(new StreamSource(new java.io.StringReader(response)));

            context.setMessage(soapMessage);

            System.out.println("Return from cache with key: "+cacheKey);
        } catch (SOAPException ignored) {
        }

        return true;
    }

    @Override
    public boolean handleFault(SOAPMessageContext context) {
        return false;
    }

    @Override
    public void close(MessageContext context) {

    }
}
