package com.kms.handler;

import com.kms.crosscut.LoggingHelper;
import com.kms.crosscut.SoapFaultHelper;
import com.sun.net.httpserver.HttpExchange;
import org.w3c.dom.Node;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Optional;
import java.util.Set;

public class LogHandler implements SOAPHandler<SOAPMessageContext> {

    @Override
    public Set<QName> getHeaders() {
        return null;
    }

    @Override
    public boolean handleMessage(SOAPMessageContext soapMessageContext) {
        final boolean messageHandled =
                (boolean) soapMessageContext.getOrDefault("messageHandled", false);

        if (messageHandled)
            return true;

        soapMessageContext.put("messageHandled", true);

        final String address = Optional
                .of((HttpExchange)soapMessageContext.get("com.sun.xml.internal.ws.http.exchange"))
                .map(HttpExchange::getRemoteAddress)
                .map(remoteAddress -> String.valueOf(remoteAddress.getAddress()))
                .orElse("");
        try {
            SOAPBody soapBody = soapMessageContext.getMessage().getSOAPBody();
            OutputStream outputStream = new ByteArrayOutputStream();
            soapMessageContext.getMessage().writeTo(outputStream);

            final String callerId = (String) soapMessageContext.getOrDefault("callerId", "null");

            // extra defensive to avoid NPE
            final String requestedServiceName = Optional.of(soapBody.getChildNodes())
                    .map(nodeList -> nodeList.item(1))
                    .map(Node::getNodeName)
                    .orElse("");

            LoggingHelper loggingHelper = LoggingHelper.getInstance();
            if (callerId.equals("null")) {
                loggingHelper.log(outputStream.toString(), address, requestedServiceName,
                        callerId, "invalid api key");

                SoapFaultHelper.throwFault("InvalidRequest","API key is invalid.");
            }
            loggingHelper.log(outputStream.toString(), address, requestedServiceName, callerId, "");

        } catch (SOAPException | IOException ignored) {
        }
        return true;
    }

    @Override
    public boolean handleFault(SOAPMessageContext soapMessageContext) {
        return false;
    }

    @Override
    public void close(MessageContext messageContext) {

    }
}
