package com.kms.service;

import com.kms.service.helper.EmailRateLimiter;
import com.kms.util.EmailUtil;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.util.Optional;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class EmailService {

    private static final ExecutorService executorService = Executors.newFixedThreadPool(5);

    private static final Session session;
    private static final Properties properties;
    private static final String SENDER_EMAIL;
    private static final String SENDER_PASSWORD;

    static {
        properties = new Properties();
        properties.put("mail.smtp.host", "smtp.gmail.com");
        properties.put("mail.smtp.socketFactory.port", "465");
        properties.put("mail.smtp.socketFactory.class",
                "javax.net.ssl.SSLSocketFactory");
        properties.put("mail.smtp.auth", "true");
        properties.put("mail.smtp.port", "465");

        SENDER_EMAIL = System.getenv("sender_email");
        SENDER_PASSWORD = System.getenv("sender_password");
        session = Session.getDefaultInstance(properties,
                new javax.mail.Authenticator() {
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(SENDER_EMAIL,SENDER_PASSWORD);
                    }
                });
    }


    public static void send(String recipient, String cc, String subject, String body){
        if (recipient == null || recipient.isEmpty() || !EmailRateLimiter.isEmailAllowed(recipient))
            return;

        try {
            MimeMessage message = new MimeMessage(session);

            int toCount = 0;
            for (String email: recipient.split(",")) {
                email = email.trim();
                if (!EmailUtil.isEmailValid(email))
                    continue;
                ++toCount;
                message.addRecipient(Message.RecipientType.TO,new InternetAddress(email));
            }
            // cancel if there is no recipient
            if (toCount == 0)
                return;

            Optional.ofNullable(cc).ifPresent(el -> {
                for (String email: el.split(",")) {
                    email = email.trim();
                    if (!EmailUtil.isEmailValid(email))
                        continue;
                    try {
                        message.addRecipient(Message.RecipientType.CC, new InternetAddress(email));
                    } catch (MessagingException e) {
                        System.out.println("Error while sending email: " + e.getMessage());
                    }
                }
            });
            message.setSubject(subject);
            message.setContent(body, "text/html");

            executorService.submit(() -> {
                try {
                    Transport.send(message);

                    System.out.println("email sent to: " + recipient);
                } catch (MessagingException ignored) {
                }
            });
        } catch (MessagingException e) {
            System.out.println("Error while sending email: " + e.getMessage());
        }
    }

}
