package com.kms.service.helper;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

public class EmailRateLimiter {

    private static final String REDIS_HOSTNAME =
            System.getenv().getOrDefault("REDIS_HOSTNAME", "localhost");
    private static final JedisPool jedisPool = new JedisPool(REDIS_HOSTNAME, 6379);
    private static final int RATE_LIMIT_PERIOD_SECONDS = 300;
    private static final int MAX_SEND_PER_PERIOD = 3;

    /**
     *
     * this function limits MAX_SEND_PER_PERIOD email sending per RATE_LIMIT_PERIOD_SECONDS per feature,
     * based on the email address.
     *
     * @param email recipient email address
     * @return status whether email sending is still allowed
     */
    public static boolean isEmailAllowed(String email) {
        try (Jedis jedis = jedisPool.getResource()){
            String key = "email:" + email;

            String emailSent = jedis.get(key);
            int emailSentInPeriod = convertValueToTryCount(emailSent);

            if (emailSentInPeriod < MAX_SEND_PER_PERIOD) {
                ++emailSentInPeriod;
                jedis.set(key, String.valueOf(emailSentInPeriod));

                if (emailSentInPeriod == 1) {
                    jedis.expire(key, RATE_LIMIT_PERIOD_SECONDS);
                }
                return true;
            }
            return false;
        }
    }

    private static int convertValueToTryCount(String number) {
        if (number == null)
            return 0;

        try {
            return Integer.parseInt(number);
        } catch (NumberFormatException ignored) {
            return 0;
        }
    }
}
